@@ -31,8 +31,10 @@ import java.sql.SQLFeatureNotSupportedException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Properties;
+import java.util.Set;
 import java.util.logging.Logger;
 
 import javax.management.InstanceAlreadyExistsException;
@@ -63,10 +65,10 @@ import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
  * @author Glenn L. Nielsen
  * @author Craig R. McClanahan
  * @author Dirk Verbeeck
- * @version $Revision: 1572244 $ $Date: 2014-02-26 20:38:08 +0000 (Wed, 26 Feb 2014) $
+ * @version $Id: BasicDataSource.java 1693836 2015-08-02 22:51:27Z psteitz $
  * @since 2.0
  */
-public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBeanRegistration {
+public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBeanRegistration, AutoCloseable {
 
     private static final Log log = LogFactory.getLog(BasicDataSource.class);
 
@@ -974,9 +976,11 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
      * @return the current number of active connections
      */
     @Override
-    public synchronized int getNumActive() {
-        if (connectionPool != null) {
-            return connectionPool.getNumActive();
+    public int getNumActive() {
+        // Copy reference to avoid NPE if close happens after null check
+        GenericObjectPool<PoolableConnection> pool = connectionPool;
+        if (pool != null) {
+            return pool.getNumActive();
         }
         return 0;
     }
@@ -989,9 +993,11 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
      * @return the current number of idle connections
      */
     @Override
-    public synchronized int getNumIdle() {
-        if (connectionPool != null) {
-            return connectionPool.getNumIdle();
+    public int getNumIdle() {
+        // Copy reference to avoid NPE if close happens after null check
+        GenericObjectPool<PoolableConnection> pool = connectionPool;
+        if (pool != null) {
+            return pool.getNumIdle();
         }
         return 0;
     }
@@ -1268,6 +1274,20 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
         return maxConnLifetimeMillis;
     }
 
+    private boolean logExpiredConnections = true;
+
+    /**
+     * When {@link #getMaxConnLifetimeMillis()} is set to limit connection lifetime,
+     * this property determines whether or not log messages are generated when the
+     * pool closes connections due to maximum lifetime exceeded.
+     *
+     * @since 2.1
+     */
+    @Override
+    public boolean getLogExpiredConnections() {
+        return logExpiredConnections;
+    }
+
     /**
      * <p>Sets the maximum permitted lifetime of a connection in
      * milliseconds. A value of zero or less indicates an infinite lifetime.</p>
@@ -1281,6 +1301,16 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
         this.maxConnLifetimeMillis = maxConnLifetimeMillis;
     }
 
+    /**
+     * When {@link #getMaxConnLifetimeMillis()} is set to limit connection lifetime,
+     * this property determines whether or not log messages are generated when the
+     * pool closes connections due to maximum lifetime exceeded.  Set this property
+     * to false to suppress log messages when connections expire.
+     */
+    public void setLogExpiredConnections(boolean logExpiredConnections) {
+        this.logExpiredConnections = logExpiredConnections;
+    }
+
     private String jmxName = null;
 
     /**
@@ -1307,9 +1337,9 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
 
     /**
      * Returns the value of the flag that controls whether or not connections
-     * being returned to the pool will checked and configured with
+     * being returned to the pool will be checked and configured with
      * {@link Connection#setAutoCommit(boolean) Connection.setAutoCommit(true)}
-     * if the auto commit setting is <code>false</false> when the connection
+     * if the auto commit setting is {@code false} when the connection
      * is returned. It is <code>true</code> by default.
      */
     public boolean getEnableAutoCommitOnReturn() {
@@ -1318,16 +1348,15 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
 
     /**
      * Sets the value of the flag that controls whether or not connections
-     * being returned to the pool will checked and configured with
+     * being returned to the pool will be checked and configured with
      * {@link Connection#setAutoCommit(boolean) Connection.setAutoCommit(true)}
-     * if the auto commit setting is <code>false</false> when the connection
+     * if the auto commit setting is {@code false} when the connection
      * is returned. It is <code>true</code> by default.
      */
     public void setEnableAutoCommitOnReturn(boolean enableAutoCommitOnReturn) {
         this.enableAutoCommitOnReturn = enableAutoCommitOnReturn;
     }
 
-
     private boolean rollbackOnReturn = true;
 
     /**
@@ -1348,6 +1377,97 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
         this.rollbackOnReturn = rollbackOnReturn;
     }
 
+    private volatile Set<String> disconnectionSqlCodes;
+
+    /**
+     * Returns the set of SQL_STATE codes considered to signal fatal conditions.
+     * @return fatal disconnection state codes
+     * @see #setDisconnectionSqlCodes(Collection)
+     * @since 2.1
+     */
+    public Set<String> getDisconnectionSqlCodes() {
+        Set<String> result = disconnectionSqlCodes;
+        if (result == null) {
+            return Collections.emptySet();
+        }
+        return result;
+    }
+
+    /**
+     * Provides the same data as {@link #getDisconnectionSqlCodes} but in an
+     * array so it is accessible via JMX.
+     * @since 2.1
+     */
+    @Override
+    public String[] getDisconnectionSqlCodesAsArray() {
+        Collection<String> result = getDisconnectionSqlCodes();
+        return result.toArray(new String[result.size()]);
+    }
+
+    /**
+     * Sets the SQL_STATE codes considered to signal fatal conditions.
+     * <p>
+     * Overrides the defaults in {@link Utils#DISCONNECTION_SQL_CODES}
+     * (plus anything starting with {@link Utils#DISCONNECTION_SQL_CODE_PREFIX}).
+     * If this property is non-null and {@link #getFastFailValidation()} is
+     * {@code true}, whenever connections created by this datasource generate exceptions
+     * with SQL_STATE codes in this list, they will be marked as "fatally disconnected"
+     * and subsequent validations will fail fast (no attempt at isValid or validation
+     * query).</p>
+     * <p>
+     * If {@link #getFastFailValidation()} is {@code false} setting this property has no
+     * effect.</p>
+     * <p>
+     * Note: this method currently has no effect once the pool has been
+     * initialized.  The pool is initialized the first time one of the
+     * following methods is invoked: {@code getConnection, setLogwriter,
+     * setLoginTimeout, getLoginTimeout, getLogWriter}.</p>
+     *
+     * @param disconnectionSqlCodes SQL_STATE codes considered to signal fatal conditions
+     * @since 2.1
+     */
+    public void setDisconnectionSqlCodes(Collection<String> disconnectionSqlCodes) {
+        if (disconnectionSqlCodes != null && disconnectionSqlCodes.size() > 0) {
+            HashSet<String> newVal = null;
+            for (String s : disconnectionSqlCodes) {
+            if (s != null && s.trim().length() > 0) {
+                    if (newVal == null) {
+                        newVal = new HashSet<>();
+                    }
+                    newVal.add(s);
+                }
+            }
+            this.disconnectionSqlCodes = newVal;
+        } else {
+            this.disconnectionSqlCodes = null;
+        }
+    }
+
+    private boolean fastFailValidation;
+
+    /**
+     * True means that validation will fail immediately for connections that
+     * have previously thrown SQLExceptions with SQL_STATE indicating fatal
+     * disconnection errors.
+     *
+     * @return true if connections created by this datasource will fast fail validation.
+     * @see #setDisconnectionSqlCodes(Collection)
+     * @since 2.1
+     */
+    @Override
+    public boolean getFastFailValidation() {
+        return fastFailValidation;
+    }
+
+    /**
+     * @see #getFastFailValidation()
+     * @param fastFailValidation true means connections created by this factory will
+     * fast fail validation
+     * @since 2.1
+     */
+    public void setFastFailValidation(boolean fastFailValidation) {
+        this.fastFailValidation = fastFailValidation;
+    }
 
     // ----------------------------------------------------- Instance Variables
 
@@ -1512,18 +1632,18 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
      * <p>Flag to remove abandoned connections if they exceed the
      * removeAbandonedTimeout when borrowObject is invoked.</p>
      *
-     * <p>The default value is false.<p>
+     * <p>The default value is false.</p>
      *
      * <p>If set to true a connection is considered abandoned and eligible
      * for removal if it has not been used for more than
      * {@link #getRemoveAbandonedTimeout() removeAbandonedTimeout} seconds.</p>
      *
      * <p>Abandoned connections are identified and removed when
-     * {@link #getConnection()} is invoked and the following conditions hold
-     * <ul><li>{@link #getRemoveAbandonedOnBorrow()} or
-     *         {@link #getRemoveAbandonedOnMaintenance()} = true</li>
-     *     <li>{@link #getNumActive()} > {@link #getMaxTotal()} - 3 </li>
-     *     <li>{@link #getNumIdle()} < 2 </li></ul></p>
+     * {@link #getConnection()} is invoked and all of the following conditions hold:
+     * </p>
+     * <ul><li>{@link #getRemoveAbandonedOnBorrow()} </li>
+     *     <li>{@link #getNumActive()} &gt; {@link #getMaxTotal()} - 3 </li>
+     *     <li>{@link #getNumIdle()} &lt; 2 </li></ul>
      *
      * @see #getRemoveAbandonedTimeout()
      */
@@ -1536,19 +1656,9 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
     }
 
     /**
-     * <p>Flag to remove abandoned connections if they exceed the
-     * removeAbandonedTimeout when borrowObject is invoked.</p>
-     *
-     * <p>If set to true a connection is considered abandoned and eligible
-     * for removal if it has been idle longer than the
-     * {@link #getRemoveAbandonedTimeout() removeAbandonedTimeout}.</p>
-     *
-     * <p>Setting this to true can recover db connections from poorly written
-     * applications which fail to close a connection.</p>
-     *
-     * @param removeAbandonedOnMaintenance true means abandoned connections will
-     *                                     be removed when borrowObject is
-     *                                     invoked
+     * @param removeAbandonedOnMaintenance true means abandoned connections may
+     *                                     be removed on pool maintenance.
+     * @see #getRemoveAbandonedOnMaintenance()
      */
     public void setRemoveAbandonedOnMaintenance(
             boolean removeAbandonedOnMaintenance) {
@@ -1563,19 +1673,12 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
      * <p>Flag to remove abandoned connections if they exceed the
      * removeAbandonedTimeout during pool maintenance.</p>
      *
-     * <p>The default value is false.<p>
+     * <p>The default value is false.</p>
      *
      * <p>If set to true a connection is considered abandoned and eligible
      * for removal if it has not been used for more than
      * {@link #getRemoveAbandonedTimeout() removeAbandonedTimeout} seconds.</p>
      *
-     * <p>Abandoned connections are identified and removed when
-     * {@link #getConnection()} is invoked and the following conditions hold
-     * <ul><li>{@link #getRemoveAbandonedOnBorrow()} or
-     *         {@link #getRemoveAbandonedOnMaintenance()} = true</li>
-     *     <li>{@link #getNumActive()} > {@link #getMaxTotal()} - 3 </li>
-     *     <li>{@link #getNumIdle()} < 2 </li></ul></p>
-     *
      * @see #getRemoveAbandonedTimeout()
      */
     @Override
@@ -1587,18 +1690,9 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
     }
 
     /**
-     * <p>Flag to remove abandoned connections if they exceed the
-     * removeAbandonedTimeout during pool maintenance.</p>
-     *
-     * <p>If set to true a connection is considered abandoned and eligible
-     * for removal if it has been idle longer than the
-     * {@link #getRemoveAbandonedTimeout() removeAbandonedTimeout}.</p>
-     *
-     * <p>Setting this to true can recover db connections from poorly written
-     * applications which fail to close a connection.</p>
-     *
-     * @param removeAbandonedOnBorrow true means abandoned connections will be
-     *                                removed during pool maintenance
+     * @param removeAbandonedOnBorrow true means abandoned connections may be
+     *                                removed when connections are borrowed from the pool.
+     * @see #getRemoveAbandonedOnBorrow()
      */
     public void setRemoveAbandonedOnBorrow(boolean removeAbandonedOnBorrow) {
         if (abandonedConfig == null) {
@@ -1614,13 +1708,13 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
      * one of these to execute a query (using one of the execute methods)
      * resets the lastUsed property of the parent connection.</p>
      *
-     * <p>Abandoned connection cleanup happens when
-     * <code><ul>
+     * <p>Abandoned connection cleanup happens when:</p>
+     * <ul>
      * <li>{@link #getRemoveAbandonedOnBorrow()} or
      *     {@link #getRemoveAbandonedOnMaintenance()} = true</li>
      * <li>{@link #getNumIdle() numIdle} &lt; 2</li>
      * <li>{@link #getNumActive() numActive} &gt; {@link #getMaxTotal() maxTotal} - 3</li>
-     * </ul></code></p>
+     * </ul>
      *
      * <p>The default value is 300 seconds.</p>
      */
@@ -1808,13 +1902,14 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
      * these connections to the pool, the underlying JDBC connections are closed.</p>
      *
      * <p>Attempts to acquire connections using {@link #getConnection()} after this method has been
-     * invoked result in SQLExceptions.<p>
+     * invoked result in SQLExceptions.</p>
      *
      * <p>This method is idempotent - i.e., closing an already closed BasicDataSource has no effect
      * and does not generate exceptions.</p>
      *
      * @throws SQLException if an error occurs closing idle connections
      */
+    @Override
     public synchronized void close() throws SQLException {
         if (registeredJmxName != null) {
             MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
@@ -1837,7 +1932,7 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
         } catch(RuntimeException e) {
             throw e;
         } catch(Exception e) {
-            throw new SQLException("Cannot close connection pool", e);
+            throw new SQLException(Utils.getMessage("pool.close.fail"), e);
         }
     }
 
@@ -1865,6 +1960,47 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
         throw new SQLFeatureNotSupportedException();
     }
 
+    /**
+     * Manually invalidates a connection, effectively requesting the pool to try
+     * to close it, remove it from the pool and reclaim pool capacity.
+     *
+     * @throws IllegalStateException
+     *             if invalidating the connection failed.
+     * @since 2.1
+     */
+    public void invalidateConnection(Connection connection) throws IllegalStateException {
+        if (connection == null) {
+            return;
+        }
+        if (connectionPool == null) {
+            throw new IllegalStateException("Cannot invalidate connection: ConnectionPool is null.");
+        }
+
+        final PoolableConnection poolableConnection;
+        try {
+            poolableConnection = connection.unwrap(PoolableConnection.class);
+            if (poolableConnection == null) {
+                throw new IllegalStateException(
+                        "Cannot invalidate connection: Connection is not a poolable connection.");
+            }
+        } catch (SQLException e) {
+            throw new IllegalStateException("Cannot invalidate connection: Unwrapping poolable connection failed.", e);
+        }
+
+        // attempt to close the connection for good measure
+        try {
+            connection.close();
+        } catch (Exception e) {
+            // ignore any exceptions here
+        }
+
+        try {
+            connectionPool.invalidateObject(poolableConnection);
+        } catch (Exception e) {
+            throw new IllegalStateException("Invalidating connection threw unexpected exception", e);
+        }
+    }
+
     // ------------------------------------------------------ Protected Methods
 
 
@@ -1921,10 +2057,11 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
             }
 
             // Create the pooling data source to manage connections
+            DataSource newDataSource; 
             success = false;
             try {
-                dataSource = createDataSourceInstance();
-                dataSource.setLogWriter(logWriter);
+                newDataSource = createDataSourceInstance();
+                newDataSource.setLogWriter(logWriter);
                 success = true;
             } catch (SQLException se) {
                 throw se;
@@ -1951,6 +2088,7 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
             // If timeBetweenEvictionRunsMillis > 0, start the pool's evictor task
             startPoolMaintenance();
 
+            dataSource = newDataSource;
             return dataSource;
         }
     }
@@ -2058,6 +2196,7 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
         // Create an object pool to contain our active connections
         GenericObjectPoolConfig config = new GenericObjectPoolConfig();
         updateJmxName(config);
+        config.setJmxEnabled(registeredJmxName != null);  // Disable JMX on the underlying pool if the DS is not registered.
         GenericObjectPool<PoolableConnection> gop;
         if (abandonedConfig != null &&
                 (abandonedConfig.getRemoveAbandonedOnBorrow() ||
@@ -2078,7 +2217,8 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
         gop.setMinEvictableIdleTimeMillis(minEvictableIdleTimeMillis);
         gop.setTestWhileIdle(testWhileIdle);
         gop.setLifo(lifo);
-        gop.setSwallowedExceptionListener(new SwallowedExceptionLogger(log));
+        gop.setSwallowedExceptionListener(new SwallowedExceptionLogger(log, logExpiredConnections));
+        gop.setEvictionPolicyClassName(evictionPolicyClassName);
         factory.setPool(gop);
         connectionPool = gop;
     }
@@ -2145,6 +2285,8 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
             connectionFactory.setRollbackOnReturn(getRollbackOnReturn());
             connectionFactory.setEnableAutoCommitOnReturn(getEnableAutoCommitOnReturn());
             connectionFactory.setDefaultQueryTimeout(getDefaultQueryTimeout());
+            connectionFactory.setFastFailValidation(fastFailValidation);
+            connectionFactory.setDisconnectionSqlCodes(disconnectionSqlCodes);
             validateConnectionFactory(connectionFactory);
         } catch (RuntimeException e) {
             throw e;
@@ -2267,4 +2409,4 @@ public class BasicDataSource implements DataSource, BasicDataSourceMXBean, MBean
             return createDataSource().getConnection();
         }
     }
-}
\ No newline at end of file
+}
