@@ -20,20 +20,42 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.pool2.SwallowedExceptionListener;
 
 /**
- * Class for logging swallowed exceptions
+ * Class for logging swallowed exceptions.
+ * @version $Id: SwallowedExceptionLogger.java 1651275 2015-01-13 04:11:09Z psteitz $
  * @since 2.0
  */
 public class SwallowedExceptionLogger implements SwallowedExceptionListener{
 
     private final Log log;
+    private final boolean logExpiredConnections;
 
+    /**
+     * Create a SwallowedExceptionLogger with the given logger.  By default,
+     * expired connection logging is turned on.
+     *
+     * @param log logger
+     */
     public SwallowedExceptionLogger(Log log) {
+        this(log, true);    
+    }
+    
+    /**
+     * Create a SwallowedExceptionLogger with the given logger and expired
+     * connection logging property.
+     *
+     * @param log logger
+     * @param logExpiredConnections false suppresses logging of expired connection events
+     */
+    public SwallowedExceptionLogger(Log log, boolean logExpiredConnections) {
         this.log = log;
+        this.logExpiredConnections = logExpiredConnections;
     }
 
     @Override
     public void onSwallowException(Exception e) {
-        log.warn(Utils.getMessage(
-                "swallowedExceptionLogger.onSwallowedException"), e);
+        if (logExpiredConnections || !(e instanceof LifetimeExceededException)) {
+            log.warn(Utils.getMessage(
+                    "swallowedExceptionLogger.onSwallowedException"), e);
+        }
     }
-}
\ No newline at end of file
+}
