@@ -24,9 +24,19 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.Hashtable;
+import java.util.Random;
 import java.util.Stack;
 
-import junit.framework.TestCase;
+import org.junit.After;
+import org.junit.Test;
+
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 // XXX FIX ME XXX
 // this class still needs some cleanup, but at least
@@ -41,16 +51,12 @@ import junit.framework.TestCase;
  * @author Sean C. Sullivan
  * @author John McNally
  * @author Dirk Verbeeck
- * @version $Revision: 1572068 $ $Date: 2014-02-26 14:51:04 +0000 (Wed, 26 Feb 2014) $
+ * @version $Id: TestConnectionPool.java 1660791 2015-02-19 04:13:10Z psteitz $
  */
-public abstract class TestConnectionPool extends TestCase {
-    public TestConnectionPool(String testName) {
-        super(testName);
-    }
+public abstract class TestConnectionPool {
 
-    @Override
+    @After
     public void tearDown() throws Exception {
-        super.tearDown();
         // Close any connections opened by the test
         while (!connections.isEmpty()) {
             Connection conn = connections.pop();
@@ -97,6 +103,7 @@ public abstract class TestConnectionPool extends TestCase {
 
     // ----------- tests ---------------------------------
 
+    @Test
     public void testClearWarnings() throws Exception {
         Connection[] c = new Connection[getMaxTotal()];
         for (int i = 0; i < c.length; i++) {
@@ -130,6 +137,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
+    @Test
     public void testIsClosed() throws Exception {
         for(int i=0;i<getMaxTotal();i++) {
             Connection conn = newConnection();
@@ -151,6 +159,7 @@ public abstract class TestConnectionPool extends TestCase {
      * Verify the close method can be called multiple times on a single connection without
      * an exception being thrown.
      */
+    @Test
     public void testCanCloseConnectionTwice() throws Exception {
         for (int i = 0; i < getMaxTotal(); i++) { // loop to show we *can* close again once we've borrowed it from the pool again
             Connection conn = newConnection();
@@ -163,6 +172,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
+    @Test
     public void testCanCloseStatementTwice() throws Exception {
         Connection conn = newConnection();
         assertNotNull(conn);
@@ -181,6 +191,7 @@ public abstract class TestConnectionPool extends TestCase {
         conn.close();
     }
 
+    @Test
     public void testCanClosePreparedStatementTwice() throws Exception {
         Connection conn = newConnection();
         assertNotNull(conn);
@@ -199,6 +210,7 @@ public abstract class TestConnectionPool extends TestCase {
         conn.close();
     }
 
+    @Test
     public void testCanCloseCallableStatementTwice() throws Exception {
         Connection conn = newConnection();
         assertNotNull(conn);
@@ -217,6 +229,7 @@ public abstract class TestConnectionPool extends TestCase {
         conn.close();
     }
 
+    @Test
     public void testCanCloseResultSetTwice() throws Exception {
         Connection conn = newConnection();
         assertNotNull(conn);
@@ -237,6 +250,7 @@ public abstract class TestConnectionPool extends TestCase {
         conn.close();
     }
 
+    @Test
     public void testBackPointers() throws Exception {
         // normal statement
         Connection conn = newConnection();
@@ -312,6 +326,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
+    @Test
     public void testSimple() throws Exception {
         Connection conn = newConnection();
         assertNotNull(conn);
@@ -325,6 +340,7 @@ public abstract class TestConnectionPool extends TestCase {
         conn.close();
     }
 
+    @Test
     public void testRepeatedBorrowAndReturn() throws Exception {
         for(int i=0;i<100;i++) {
             Connection conn = newConnection();
@@ -340,6 +356,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
+    @Test
     public void testSimple2() throws Exception {
         Connection conn = newConnection();
         assertNotNull(conn);
@@ -392,6 +409,7 @@ public abstract class TestConnectionPool extends TestCase {
         conn = null;
     }
 
+    @Test
     public void testPooling() throws Exception {
         // Grab a maximal set of open connections from the pool
         Connection[] c = new Connection[getMaxTotal()];
@@ -427,6 +445,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
+    @Test
     public void testAutoCommitBehavior() throws Exception {
         Connection conn0 = newConnection();
         assertNotNull("connection should not be null", conn0);
@@ -452,7 +471,8 @@ public abstract class TestConnectionPool extends TestCase {
         conn3.close();
     }
 
-    /** @see "http://issues.apache.org/bugzilla/show_bug.cgi?id=12400" */
+    /** "http://issues.apache.org/bugzilla/show_bug.cgi?id=12400" */
+    @Test
     public void testConnectionsAreDistinct() throws Exception {
         Connection[] conn = new Connection[getMaxTotal()];
         for(int i=0;i<conn.length;i++) {
@@ -467,7 +487,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
-
+    @Test
     public void testOpening() throws Exception {
         Connection[] c = new Connection[getMaxTotal()];
         // test that opening new connections is not closing previous
@@ -484,6 +504,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
+    @Test
     public void testClosing() throws Exception {
         Connection[] c = new Connection[getMaxTotal()];
         // open the maximum connections
@@ -503,6 +524,7 @@ public abstract class TestConnectionPool extends TestCase {
         }
     }
 
+    @Test
     public void testMaxTotal() throws Exception {
         Connection[] c = new Connection[getMaxTotal()];
         for (int i = 0; i < c.length; i++) {
@@ -527,6 +549,7 @@ public abstract class TestConnectionPool extends TestCase {
      * DBCP-128: BasicDataSource.getConnection()
      * Connections don't work as hashtable keys
      */
+    @Test
     public void testHashing() throws Exception {
         Connection con = getConnection();
         Hashtable<Connection, String> hash = new Hashtable<>();
@@ -538,6 +561,7 @@ public abstract class TestConnectionPool extends TestCase {
         con.close();
     }
 
+    @Test
     public void testThreaded() {
         TestThread[] threads = new TestThread[getMaxTotal()];
         for(int i=0;i<threads.length;i++) {
@@ -617,6 +641,7 @@ public abstract class TestConnectionPool extends TestCase {
     // Bugzilla Bug 24328: PooledConnectionImpl ignores resultsetType
     // and Concurrency if statement pooling is not enabled
     // http://issues.apache.org/bugzilla/show_bug.cgi?id=24328
+    @Test
     public void testPrepareStatementOptions() throws Exception
     {
         Connection conn = newConnection();
@@ -638,6 +663,7 @@ public abstract class TestConnectionPool extends TestCase {
 
     // Bugzilla Bug 24966: NullPointer with Oracle 9 driver
     // wrong order of passivate/close when a rset isn't closed
+    @Test
     public void testNoRsetClose() throws Exception {
         Connection conn = newConnection();
         assertNotNull(conn);
@@ -651,6 +677,7 @@ public abstract class TestConnectionPool extends TestCase {
     }
 
     // Bugzilla Bug 26966: Connectionpool's connections always returns same
+    @Test
     public void testHashCode() throws Exception {
         Connection conn1 = newConnection();
         assertNotNull(conn1);
@@ -694,9 +721,9 @@ public abstract class TestConnectionPool extends TestCase {
 
     /**
      * Launches a group of 2 * getMaxTotal() threads, each of which will attempt to obtain a connection
-     * from the pool, hold it for <holdTime> ms, and then return it to the pool.  If <loopOnce> is false,
-     * threads will continue this process indefinitely.  If <expectError> is true, exactly 1/2 of the
-     * threads are expected to either throw exceptions or fail to complete. If <expectError> is false,
+     * from the pool, hold it for {@code holdTime} ms, and then return it to the pool.  If {@code loopOnce} is false,
+     * threads will continue this process indefinitely.  If {@code expectError} is true, exactly 1/2 of the
+     * threads are expected to either throw exceptions or fail to complete. If {@code expectError} is false,
      * all threads are expected to complete successfully.
      *
      * @param holdTime time in ms that a thread holds a connection before returning it to the pool
@@ -706,11 +733,33 @@ public abstract class TestConnectionPool extends TestCase {
      *
      * @throws Exception
      */
+    protected void multipleThreads(final int holdTime,
+        final boolean expectError, final boolean loopOnce,
+        final long maxWaitMillis) throws Exception {
+        multipleThreads(holdTime, expectError, loopOnce, maxWaitMillis, 1, 2 * getMaxTotal(), 300);
+    }
+    
+    /**
+     * Launches a group of {@code numThreads} threads, each of which will attempt to obtain a connection
+     * from the pool, hold it for {@code holdTime} ms, and then return it to the pool.  If {@code loopOnce} is false,
+     * threads will continue this process indefinitely.  If {@code expectError} is true, exactly 1/2 of the
+     * threads are expected to either throw exceptions or fail to complete. If {@code expectError} is false,
+     * all threads are expected to complete successfully.  Threads are stopped after {@code duration} ms.
+     *
+     * @param holdTime time in ms that a thread holds a connection before returning it to the pool
+     * @param expectError whether or not an error is expected
+     * @param loopOnce whether threads should complete the borrow - hold - return cycle only once, or loop indefinitely
+     * @param maxWaitMillis passed in by client - has no impact on the test itself, but does get reported
+     * @param numThreads the number of threads
+     * @param duration duration in ms of test
+     *
+     * @throws Exception
+     */
     protected void multipleThreads(final int holdTime,
             final boolean expectError, final boolean loopOnce,
-            final long maxWaitMillis) throws Exception {
+            final long maxWaitMillis, int numStatements, int numThreads, long duration) throws Exception {
                 long startTime = timeStamp();
-                final PoolTest[] pts = new PoolTest[2 * getMaxTotal()];
+                final PoolTest[] pts = new PoolTest[numThreads];
                 // Catch Exception so we can stop all threads if one fails
                 ThreadGroup threadGroup = new ThreadGroup("foo") {
                     @Override
@@ -722,7 +771,7 @@ public abstract class TestConnectionPool extends TestCase {
                 };
                 // Create all the threads
                 for (int i = 0; i < pts.length; i++) {
-                    pts[i] = new PoolTest(threadGroup, holdTime, expectError, loopOnce);
+                    pts[i] = new PoolTest(threadGroup, holdTime, expectError, loopOnce, numStatements);
                 }
                 // Start all the threads
                 for (PoolTest pt : pts) {
@@ -730,7 +779,7 @@ public abstract class TestConnectionPool extends TestCase {
                 }
 
                 // Give all threads a chance to start and succeed
-                Thread.sleep(300L);
+                Thread.sleep(duration);
 
                 // Stop threads
                 for (PoolTest pt : pts) {
@@ -817,6 +866,8 @@ public abstract class TestConnectionPool extends TestCase {
          * The number of milliseconds to hold onto a database connection
          */
         private final int connHoldTime;
+        
+        private final int numStatements;
 
         private volatile boolean isRun;
 
@@ -825,6 +876,8 @@ public abstract class TestConnectionPool extends TestCase {
         private final Thread thread;
 
         private Throwable thrown;
+        
+        private final Random random = new Random();
 
         // Debug for DBCP-318
         private final long created; // When object was created
@@ -841,10 +894,14 @@ public abstract class TestConnectionPool extends TestCase {
         private final boolean loopOnce; // If true, don't repeat loop
 
         public PoolTest(ThreadGroup threadGroup, int connHoldTime, boolean isStopOnException) {
-            this(threadGroup, connHoldTime, isStopOnException, false);
+            this(threadGroup, connHoldTime, isStopOnException, false, 1);
+        }
+        
+        public PoolTest(ThreadGroup threadGroup, int connHoldTime, boolean isStopOnException, int numStatements) {
+            this(threadGroup, connHoldTime, isStopOnException, false, numStatements);
         }
 
-        private PoolTest(ThreadGroup threadGroup, int connHoldTime, boolean isStopOnException, boolean once) {
+        private PoolTest(ThreadGroup threadGroup, int connHoldTime, boolean isStopOnException, boolean once, int numStatements) {
             this.loopOnce = once;
             this.connHoldTime = connHoldTime;
             stopOnException = isStopOnException;
@@ -854,6 +911,7 @@ public abstract class TestConnectionPool extends TestCase {
                 new Thread(threadGroup, this, "Thread+" + currentThreadCount++);
             thread.setDaemon(false);
             created = timeStamp();
+            this.numStatements = numStatements;
         }
 
         public void start(){
@@ -873,8 +931,9 @@ public abstract class TestConnectionPool extends TestCase {
                     connected = timeStamp();
                     state = "Using Connection";
                     assertNotNull(conn);
+                    final String sql = numStatements == 1 ? "select * from dual" : "select count " + random.nextInt(numStatements - 1);
                     PreparedStatement stmt =
-                        conn.prepareStatement("select * from dual");
+                        conn.prepareStatement(sql);
                     assertNotNull(stmt);
                     ResultSet rset = stmt.executeQuery();
                     assertNotNull(rset);
@@ -916,4 +975,4 @@ public abstract class TestConnectionPool extends TestCase {
     long timeStamp() {
         return System.currentTimeMillis();// JVM 1.5+ System.nanoTime() / 1000000;
     }
-}
\ No newline at end of file
+}
