@@ -17,42 +17,47 @@
 
 package org.apache.commons.dbcp2;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+
 import java.sql.CallableStatement;
 import java.sql.Connection;
 
-import junit.framework.TestCase;
+import org.junit.Before;
+import org.junit.Test;
 
 /**
- * @version $Revision: 1587109 $ $Date: 2014-04-13 15:07:42 -0700 (Sun, 13 Apr 2014) $
+ * @version $Id: TestDelegatingCallableStatement.java 1649424 2015-01-04 21:15:33Z tn $
  */
-public class TestDelegatingCallableStatement extends TestCase {
-    public TestDelegatingCallableStatement(String testName) {
-        super(testName);
-    }
+public class TestDelegatingCallableStatement {
 
     private DelegatingConnection<Connection> conn = null;
     private Connection delegateConn = null;
     private DelegatingCallableStatement stmt = null;
     private CallableStatement delegateStmt = null;
 
-    @Override
+    @Before
     public void setUp() throws Exception {
         delegateConn = new TesterConnection("test", "test");
         conn = new DelegatingConnection<>(delegateConn);
     }
 
+    @Test
     public void testExecuteQueryReturnsNull() throws Exception {
         delegateStmt = new TesterCallableStatement(delegateConn,"null");
         stmt = new DelegatingCallableStatement(conn,delegateStmt);
         assertNull(stmt.executeQuery());
     }
 
+    @Test
     public void testExecuteQueryReturnsNotNull() throws Exception {
         delegateStmt = new TesterCallableStatement(delegateConn,"select * from foo");
         stmt = new DelegatingCallableStatement(conn,delegateStmt);
         assertTrue(null != stmt.executeQuery());
     }
 
+    @Test
     public void testGetDelegate() throws Exception {
         delegateStmt = new TesterCallableStatement(delegateConn,"select * from foo");
         stmt = new DelegatingCallableStatement(conn,delegateStmt);
