@@ -17,6 +17,11 @@
 
 package org.apache.commons.dbcp2.datasources;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -30,16 +35,15 @@ import org.apache.commons.dbcp2.DelegatingStatement;
 import org.apache.commons.dbcp2.TestConnectionPool;
 import org.apache.commons.dbcp2.TesterDriver;
 import org.apache.commons.dbcp2.cpdsadapter.DriverAdapterCPDS;
+import org.junit.Before;
+import org.junit.Test;
 
 /**
  * @author John McNally
  * @author Dirk Verbeeck
- * @version $Revision: 1587109 $ $Date: 2014-04-13 15:07:42 -0700 (Sun, 13 Apr 2014) $
+ * @version $Id: TestSharedPoolDataSource.java 1660791 2015-02-19 04:13:10Z psteitz $
  */
 public class TestSharedPoolDataSource extends TestConnectionPool {
-    public TestSharedPoolDataSource(String testName) {
-        super(testName);
-    }
 
     @Override
     protected Connection getConnection() throws Exception {
@@ -49,9 +53,8 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     private DriverAdapterCPDS pcds;
     private DataSource ds;
 
-    @Override
+    @Before
     public void setUp() throws Exception {
-        super.setUp();
         pcds = new DriverAdapterCPDS();
         pcds.setDriver("org.apache.commons.dbcp2.TesterDriver");
         pcds.setUrl("jdbc:apache:commons:testdriver");
@@ -73,14 +76,11 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
 
 
     /**
-     * Switching 'u1 -> 'u2' and 'p1' -> 'p2' will
-     * exhibit the bug detailed in
-     * http://issues.apache.org/bugzilla/show_bug.cgi?id=18905
-     *
      * Starting with a successful connection, then incorrect password,
      * then correct password for same user illustrates
      * JIRA: DBCP-245
      */
+    @Test
     public void testIncorrectPassword() throws Exception {
 
         ds.getConnection("u2", "p2").close();
@@ -118,6 +118,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
 
 
     @Override
+    @Test
     public void testSimple() throws Exception
     {
         Connection conn = ds.getConnection();
@@ -132,6 +133,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         conn.close();
     }
 
+    @Test
     public void testSimpleWithUsername() throws Exception
     {
         Connection conn = ds.getConnection("u1", "p1");
@@ -146,6 +148,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         conn.close();
     }
 
+    @Test
     public void testClosingWithUserName()
         throws Exception
     {
@@ -177,6 +180,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     }
 
     @Override
+    @Test
     public void testSimple2()
         throws Exception
     {
@@ -231,6 +235,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     }
 
     @Override
+    @Test
     public void testOpening()
         throws Exception
     {
@@ -252,6 +257,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     }
 
     @Override
+    @Test
     public void testClosing()
         throws Exception
     {
@@ -279,6 +285,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
      *
      * @throws Exception
      */
+    @Test
     public void testClosePool() throws Exception {
       ((SharedPoolDataSource)ds).close();
       SharedPoolDataSource tds = new SharedPoolDataSource();
@@ -287,6 +294,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     }
 
     @Override
+    @Test
     public void testMaxTotal() throws Exception {
         Connection[] c = new Connection[getMaxTotal()];
         for (int i=0; i<c.length; i++) {
@@ -306,6 +314,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         }
     }
 
+    @Test
     public void testMaxWaitMillis() throws Exception {
         final int maxWaitMillis = 1000;
         final int theadCount = 20;
@@ -350,6 +359,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         }
     }
 
+    @Test
     public void testMultipleThreads1() throws Exception {
         // Override wait time in order to allow for Thread.sleep(1) sometimes taking a lot longer on
         // some JVMs, e.g. Windows.
@@ -358,12 +368,14 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         multipleThreads(1, false, false, defaultMaxWaitMillis);
     }
 
+    @Test
     public void testMultipleThreads2() throws Exception {
         final int defaultMaxWaitMillis = 500;
         ((SharedPoolDataSource) ds).setDefaultMaxWaitMillis(defaultMaxWaitMillis);
         multipleThreads(2 * defaultMaxWaitMillis, true, true, defaultMaxWaitMillis);
     }
 
+    @Test
     public void testTransactionIsolationBehavior() throws Exception {
         Connection conn = getConnection();
         assertNotNull(conn);
@@ -385,6 +397,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
 
     // Bugzilla Bug 24136 ClassCastException in DriverAdapterCPDS
     // when setPoolPreparedStatements(true)
+    @Test
     public void testPoolPrepareStatement() throws Exception
     {
         pcds.setPoolPreparedStatements(true);
@@ -521,7 +534,10 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         assertTrue(l3HashCode == l4HashCode);
         conn.close();
         conn = null;
+        tds.close();
     }
+
+    @Test
     public void testPoolPreparedStatements() throws Exception {
         doTestPoolPreparedStatements(new PscbString());
         doTestPoolPreparedStatements(new PscbStringIntInt());
@@ -532,6 +548,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
     }
 
     // See DBCP-8
+    @Test
     public void testChangePassword() throws Exception {
         try (Connection c = ds.getConnection("foo", "bay")){
             fail("Should have generated SQLException");
@@ -566,6 +583,7 @@ public class TestSharedPoolDataSource extends TestConnectionPool {
         }
     }
 
+    @Test
     public void testDbcp369() {
         final ArrayList<SharedPoolDataSource> dataSources = new ArrayList<>();
         for (int j = 0; j < 10000; j++) {
