@@ -42,7 +42,7 @@ import javax.transaction.Transaction;
  * TestSuite for ManagedDataSource with an active transaction in progress.
  *
  * @author Dain Sundstrom
- * @version $Id: TestManagedDataSourceInTx.java 1649424 2015-01-04 21:15:33Z tn $
+ * @version $Id: TestManagedDataSourceInTx.java 1691861 2015-07-19 23:15:15Z psteitz $
  */
 public class TestManagedDataSourceInTx extends TestManagedDataSource {
 
@@ -70,6 +70,12 @@ public class TestManagedDataSourceInTx extends TestManagedDataSource {
         // two connections to the same datasource are supposed to share
         // a single connection
     }
+    
+    @Override
+    @Test
+    public void testNestedConnections() {
+        // Not supported
+    }
 
     @Override
     @Test
